/*
  # ObjectManager.java
  # $Id: ObjectManager.java,v 1.23 2002/06/08 08:36:34 yoshikia Exp $
 */

package rescueAgent.ObjectSet;

import java.util.*;
import rescueAgent.RescueObject.*;
import rescueAgent.RescueLib.*;
import rescueAgent.AreaMap.*;

/**
 * ֥ȾŪ˰Υ᥽åɷ饹<BR>
 * This class handle Rescue Object Information for decide action.
 **/
public class ObjectManager{
  /** ֥ȾǼǡ١饹 <BR>
   *  (Database of all object information)
   ***/
  protected ObjectDatabase objectDB;
  /** ϩǼ뤿Υ饹 <BR>
   *  (Class for routing information)
   **/
  protected RoutingTable routingTable;
  /** SenseϤ뤿Υ饹 <BR>
   * @deprecated 侩Ȥˤ侩Ȥʤ褦ˤ뤳ȡ
   *  (Class for parsing sense information)
   **/
  protected SenseInfoParser bodyInfoParser;

  /** ǿSenseˤ륪֥ȤID <BR>
   * @deprecated 侩Ȥˤ侩Ȥʤ褦ˤ뤳ȡ
   **/
  protected Vector crtRcvObject;

  /********************************************************************/
  // 󥹥ȥ饯

  public ObjectManager(int id){
    objectDB = new ObjectDatabase(id);
    routingTable = new RoutingTable(objectDB);
    bodyInfoParser = new SenseInfoParser(objectDB);
    // hearInfoParser = new HearInfoParser_OLD();
  }

  // 󥹥ȥ饯 ޤ
  /********************************************************************/

  public ObjectDatabase getObjectDatabase(){
    return objectDB;
  }

  /********************************************************************/

  public SenseInfoParser getSenseInfoParser(){
    return bodyInfoParser;
  }

  /********************************************************************/
  /**
   * Civilian֥ȡƤΥȡˤxɸ׻ <BR>
   * (Calculate Civilian's Object x coordinate)
   *<BR><BR>
   * @param obj 
   * @return Ϥ줿Ȥxɸ
   **/
  public double getPosX(Civilian obj){
    int srcPosID = obj.getPosition();
    double srcX = 0;

    // Υå
    if(objectDB.hasNodeObject(srcPosID)){ // object in Node
      Node tnd = objectDB.getNodeObject(srcPosID);
      srcX = tnd.getX();
    }
    else if(objectDB.hasRoadObject(srcPosID)){ // object in Road 
      Road trd = objectDB.getRoadObject(srcPosID);
      Node headNode = objectDB.getNodeObject(trd.getHead());
      Node tailNode = objectDB.getNodeObject(trd.getTail());

      if(headNode != null && tailNode != null){
	if(true){ // 濴ɸ
	  srcX = (headNode.getX() + tailNode.getX())/2;
	}
	else{
	  srcX = calcPosX(headNode.getX(),headNode.getY(),
			  tailNode.getX(),tailNode.getY(),
			  obj.getPositionExtra());
	};
      }
      else{
	//System.err.println("GetPosX H or T fail !!");
	Node tnd2 = (Node)objectDB.getNodeObject(trd.getHead());
	srcX = tnd2.getX();
      };
    }
    else if(objectDB.hasBuildingObject(srcPosID) ||
	    objectDB.hasRefugeObject(srcPosID) ||
	    objectDB.hasFireStationObject(srcPosID) ||
	    objectDB.hasAmbulanceCenterObject(srcPosID) ||
	    objectDB.hasPoliceOfficeObject(srcPosID)){
      // object in Building etc 
      Building bld = (Building)objectDB.getObject(srcPosID);
      srcX = bld.getX();
    }
    else{ // ʬΤȤ狼ʤ
      //System.err.print("ObjectManager.getPosX Error !!");
    };
    //  ޤ
    return srcX;
  }

  /********************************************************************/
  /**
   * Civilian֥ȡƤΥȡˤyɸ׻ <BR>
   * (Calculate Civilian's Object y coordinate)
   *<BR><BR>
   * @param obj 
   * @return Ϥ줿Ȥyɸ
   **/
  public double getPosY(Civilian obj){
    int srcPosID = obj.getPosition();
    double srcY = 0;

    // Υå
    if(objectDB.hasNodeObject(srcPosID)){ // ʬΤȤNode
      Node tnd = objectDB.getNodeObject(srcPosID);
      srcY = tnd.getY();
    }
    else if(objectDB.hasRoadObject(srcPosID)){ // ʬΤȤRoad 
      Road trd = objectDB.getRoadObject(srcPosID);
      Node headNode = objectDB.getNodeObject(trd.getHead());
      Node tailNode = objectDB.getNodeObject(trd.getTail());

      if(headNode != null && tailNode != null){
	if(true){
	  srcY = (headNode.getY() + tailNode.getY())/2;
	}
	else{
	  srcY = calcPosY(headNode.getX(),headNode.getY(),
			  tailNode.getX(),tailNode.getY(),
			  obj.getPositionExtra());
	};
      }
      else{
	//System.err.println("GetPosY H or T fail !!");
	Node tnd2 = objectDB.getNodeObject(trd.getHead());
	srcY = tnd2.getY();
      };
    }
    else if(objectDB.hasBuildingObject(srcPosID) ||
	    objectDB.hasRefugeObject(srcPosID) ||
	    objectDB.hasFireStationObject(srcPosID) ||
	    objectDB.hasAmbulanceCenterObject(srcPosID) ||
	    objectDB.hasPoliceOfficeObject(srcPosID)){
      // object in Building etc 
      Building bld = (Building)objectDB.getObject(srcPosID);
      srcY = bld.getY();
    }
    else{ // ʬΤȤ狼ʤ
      //System.err.print("ObjectManager.getPosY Error !!");
    };
    //  ޤ
    return srcY;
  }

  /********************************************************************/
  /**
   * Ϥ줿2(X1,Y1),(X2,Y2)εΥ֤<BR>
   *<BR><BR>
   * @param X1 xɸ
   * @param Y1 yɸ
   * @param X2 xɸ
   * @param Y2 yɸ
   * @return ֤εΥ
   **/
  public double getDistance(double X1,double Y1,double X2,double Y2){

    return Math.sqrt((X2-X1)*(X2-X1)+(Y2-Y1)*(Y2-Y1));
  }

  /********************************************************************/
  /**
   * Ϥ줿2֤ľ(x1,y1)ΥlǤxɸ֤
   * 䤳᥽åɤְ㤤ʤǧ줿(04-04-29)
   * ToDo:äƤǤääȴñ˵ޤ뵤ġ<BR>
   * (Return x coordinate which is distance l from (x1,y1) between (x1,y1) 
   * and (x2,y2))
   *<BR><BR>
   * @param X1 xɸ
   * @param Y1 yɸ
   * @param X2 xɸ
   * @param Y2 yɸ
   * @param l  εΥ
   * @return ľǵΥlȤʤxɸ
   **/
  public static double calcPosX(double X1,double Y1,
				double X2,double Y2,double l){ 
    return X1 + l * Math.cos(Math.atan((Y2 - Y1)/(X2 - X1)));
  }

  /********************************************************************/
  /**
   * Ϥ줿2֤ľ(x1,y1)ΥlǤyɸ֤
   * 䤳᥽åɤְ㤤ʤǧ줿(04-04-29)
   * ToDo:äƤǤääȴñ˵ޤ뵤ġ<BR>
   * (Return y coordinate which is distance l from (x1,y1) between (x1,y1) 
   * and (x2,y2))
   *<BR><BR>
   * @param X1 xɸ
   * @param Y1 yɸ
   * @param X2 xɸ
   * @param Y2 yɸ
   * @param l  εΥ
   * @return ľǵΥlȤʤyɸ
   **/
  public static double calcPosY(double X1,double Y1,
				double X2,double Y2,double l){
    return Y1 + l * Math.sin(Math.atan((Y2 - Y1)/(X2 - X1)));
  }

  /********************************************************************/
  /**
   * ObjectManagerObjectDatabaseϡ<BR>
   * (Output all Database's information)
   *<BR><BR>
   **/
  public void printDB(){
    objectDB.printTable();
  }

  /********************************************************************/
  /********************************************************************/
  // ObjectDatabaseѤ᥽åɷ
  // 
  /********************************************************************/
  /**
   * Ϥ줿IDBuilding֥Ȥ¸ߤ뤫å
   * Buildingϻ(FS,AC,PO)ޤǤ뤳Ȥ!!
   *<BR><BR>
   * @param id ֥ID
   * @return true=¸ߤ
   **/
  public boolean hasBuildingObject(int id){
    return objectDB.hasBuildingObject(id);
  }

  /********************************************************************/
  /**
   * Ϥ줿IDNode֥Ȥ¸ߤ뤫å
   *<BR><BR>
   * @param id ֥ID
   * @return true=¸ߤ
   **/
  public boolean hasNodeObject(int id){
    return objectDB.hasNodeObject(id);
  }

  /********************************************************************/
  /**
   * Ϥ줿IDRoad֥Ȥ¸ߤ뤫å
   *<BR><BR>
   * @param id ֥ID
   * @return true=¸ߤ
   **/
  public boolean hasRoadObject(int id){
    return objectDB.hasRoadObject(id);
  }

  /********************************************************************/
  /**
   * Ϥ줿IDCivilian֥Ȥ¸ߤ뤫å
   * Civilianϵ߽(FB,AT,PF)ޤǤ뤳Ȥ!!
   *<BR><BR>
   * @param id ֥ID
   * @return true=¸ߤ
   **/
  public boolean hasCivilianObject(int id){
    return objectDB.hasCivilianObject(id);
  }

  /********************************************************************/
  /**
   * Ϥ줿IDFireBrigade֥Ȥ¸ߤ뤫å
   *<BR><BR>
   * @param id ֥ID
   * @return true=¸ߤ
   **/
  public boolean hasFireBrigadeObject(int id){
    return objectDB.hasFireBrigadeObject(id);
  }

  /********************************************************************/
  /**
   * Ϥ줿IDAmbulanceTeam֥Ȥ¸ߤ뤫å
   *<BR><BR>
   * @param id ֥ID
   * @return true=¸ߤ
   **/
  public boolean hasAmbulanceTeamObject(int id){
    return objectDB.hasAmbulanceTeamObject(id);
  }

  /********************************************************************/
  /**
   * Ϥ줿IDPoliceForce֥Ȥ¸ߤ뤫å
   *<BR><BR>
   * @param id ֥ID
   * @return true=¸ߤ
   **/
  public boolean hasPoliceForceObject(int id){
    return objectDB.hasPoliceForceObject(id);
  }

  /********************************************************************/
  /**
   * Ϥ줿IDRefuge֥Ȥ¸ߤ뤫å
   *<BR><BR>
   * @param id ֥ID
   * @return true=¸ߤ
   **/
  public boolean hasRefugeObject(int id){
    return objectDB.hasRefugeObject(id);
  }

  /********************************************************************/
  /**
   * Ϥ줿IDFireStation֥Ȥ¸ߤ뤫å
   *<BR><BR>
   * @param id ֥ID
   * @return true=¸ߤ
   **/
  public boolean hasFireStationObject(int id){
    return objectDB.hasFireStationObject(id);
  }

  /********************************************************************/
  /**
   * Ϥ줿IDAmbulanceCenter֥Ȥ¸ߤ뤫å
   *<BR><BR>
   * @param id ֥ID
   * @return true=¸ߤ
   **/
  public boolean hasAmbulanceCenterObject(int id){
    return objectDB.hasAmbulanceCenterObject(id);
  }

  /********************************************************************/
  /**
   * Ϥ줿IDPoliceOffice֥Ȥ¸ߤ뤫å
   *<BR><BR>
   * @param id ֥ID
   * @return true=¸ߤ
   **/
  public boolean hasPoliceOfficeObject(int id){
    return objectDB.hasPoliceOfficeObject(id);
  }

  /********************************************************************/
  /**
   * Building֥Ȥ֤
   * Buildingϻ(FS,AC,PO)ޤǤ뤳Ȥ!!
   *<BR><BR>
   * @return BuildingNum ʪ
   **/
  public int getBuildingObjectNum(){
    return objectDB.getBuildingObjectNum();
  }

  /********************************************************************/
  /**
   * Node֥Ȥ֤
   *<BR><BR>
   * @return NodeNum 
   **/
  public int getNodeObjectNum(){
    return objectDB.getNodeObjectNum();
  }

  /********************************************************************/
  /**
   * Road֥Ȥ֤
   *<BR><BR>
   * @return RoadNum ƻϩ
   **/
  public int getRoadObjectNum(){
    return objectDB.getRoadObjectNum();
  }

  /********************************************************************/
  /**
   * Civilian֥Ȥ֤
   * Civilianϵ߽(FB,AT,PF)ޤǤ뤳Ȥ!!
   *<BR><BR>
   * @return CivilianNum ̱(߽ޤ)
   **/
  public int getCivilianObjectNum(){
    return objectDB.getCivilianObjectNum();
  }

  /********************************************************************/
  /**
   * FireBrigade֥Ȥ֤
   *<BR><BR>
   * @return FireBrigadeNum 
   **/
  public int getFireBrigadeObjectNum(){
    return objectDB.getFireBrigadeObjectNum();
  }

  /********************************************************************/
  /**
   * AmbulanceTeam֥Ȥ֤
   *<BR><BR>
   * @return AmbulanceTeamNum ߵ
   **/
  public int getAmbulanceTeamObjectNum(){
    return objectDB.getAmbulanceTeamObjectNum();
  }

  /********************************************************************/
  /**
   * PoliceForce֥Ȥ֤
   *<BR><BR>
   * @return PoliceForceNum ٻ()
   **/
  public int getPoliceForceObjectNum(){
    return objectDB.getPoliceForceObjectNum();
  }

  /********************************************************************/
  /**
   * Refuge֥Ȥ֤
   *<BR><BR>
   * @return RefugeNum 
   **/
  public int getRefugeObjectNum(){
    return objectDB.getRefugeObjectNum();
  }

  /********************************************************************/
  /**
   * FireStation֥Ȥ֤
   *<BR><BR>
   * @return FireStationNum ɽ
   **/
  public int getFireStationObjectNum(){
    return objectDB.getFireStationObjectNum();
  }

  /********************************************************************/
  /**
   * AmbulanceCenter֥Ȥ֤
   *<BR><BR>
   * @return AmbulanceCenterNum ߵ޽
   **/
  public int getAmbulanceCenterObjectNum(){
    return objectDB.getAmbulanceCenterObjectNum();
  }

  /********************************************************************/
  /**
   * PoliceOffice֥Ȥ֤
   *<BR><BR>
   * @return PoliceOfficeNum ٻ
   **/
  public int getPoliceOfficeObjectNum(){
    return objectDB.getPoliceOfficeObjectNum();
  }

  /********************************************************************/
  /**
   * Ϥ줿IDBuilding֥Ȥ֤
   * Buildingϻ(FS,AC,PO)ޤǤ뤳Ȥ!!
   *<BR><BR>
   * @param id ֥ID
   * @return Building IDб֥
   **/
  public Building getBuildingObject(int id){
    return objectDB.getBuildingObject(id);
  }

  /********************************************************************/
  /**
   * Ϥ줿IDNode֥Ȥ֤
   *<BR><BR>
   * @param id ֥ID
   * @return Node IDб֥
   **/
  public Node getNodeObject(int id){
    return objectDB.getNodeObject(id);
  }

  /********************************************************************/
  /**
   * Ϥ줿IDRoad֥Ȥ֤
   *<BR><BR>
   * @param id ֥ID
   * @return Road IDб֥
   **/
  public Road getRoadObject(int id){
    return objectDB.getRoadObject(id);
  }

  /********************************************************************/
  /**
   * Ϥ줿IDCivilian֥Ȥ֤
   * Civilianϵ߽(FB,AT,PF)ޤǤ뤳Ȥ!!
   *<BR><BR>
   * @param id ֥ID
   * @return Civilian IDб֥
   **/
  public Civilian getCivilianObject(int id){
    return objectDB.getCivilianObject(id);
  }

  /********************************************************************/
  /**
   * Ϥ줿IDFireBrigade֥Ȥ֤
   *<BR><BR>
   * @param id ֥ID
   * @return FireBrigade IDб֥
   **/
  public FireBrigade getFireBrigadeObject(int id){
    return objectDB.getFireBrigadeObject(id);
  }

  /********************************************************************/
  /**
   * Ϥ줿IDAmbulanceTeam֥Ȥ֤
   *<BR><BR>
   * @param id ֥ID
   * @return AmbulanceTeam IDб֥
   **/
  public AmbulanceTeam getAmbulanceTeamObject(int id){
    return objectDB.getAmbulanceTeamObject(id);
  }

  /********************************************************************/
  /**
   * Ϥ줿IDPoliceForce֥Ȥ֤
   *<BR><BR>
   * @param id ֥ID
   * @return PoliceForce IDб֥
   **/
  public PoliceForce getPoliceForceObject(int id){
    return objectDB.getPoliceForceObject(id);
  }

  /********************************************************************/
  /**
   * Ϥ줿IDRefuge֥Ȥ֤
   *<BR><BR>
   * @param id ֥ID
   * @return Refuge IDб֥
   **/
  public Refuge getRefugeObject(int id){
    return objectDB.getRefugeObject(id);
  }

  /********************************************************************/
  /**
   * Ϥ줿IDFireStation֥Ȥ֤
   *<BR><BR>
   * @param objectID ֥ID
   * @return FireStation IDб֥
   **/
  public FireStation getFireStationObject(int id){
    return objectDB.getFireStationObject(id);
  }

  /********************************************************************/
  /**
   * Ϥ줿IDAmbulanceCenter֥Ȥ֤
   *<BR><BR>
   * @param id ֥ID
   * @return AmbulanceCenter IDб֥
   **/
  public AmbulanceCenter getAmbulanceCenterObject(int id){
    return objectDB.getAmbulanceCenterObject(id);
  }

  /********************************************************************/
  /**
   * Ϥ줿IDPoliceOffice֥Ȥ֤
   *<BR><BR>
   * @param id ֥ID
   * @return PoliceOffice IDб֥
   **/
  public PoliceOffice getPoliceOfficeObject(int id){
    return objectDB.getPoliceOfficeObject(id);
  }

  /********************************************************************/
  /**
   * BuildingHashtable򤽤Τޤ֤
   *<BR><BR>
   * @return BuildingHashtable
   **/
  public Hashtable getBuildingTable(){
    return objectDB.getBuildingTable();
  }

  /********************************************************************/
  /**
   * NodeHashtable򤽤Τޤ֤
   *<BR><BR>
   * @return NodeHashtable
   **/
  public Hashtable getNodeTable(){
    return objectDB.getNodeTable();
  }

  /********************************************************************/
  /**
   * RoadHashtable򤽤Τޤ֤
   *<BR><BR>
   * @return RoadHashtable
   **/
  public Hashtable getRoadTable(){
    return objectDB.getRoadTable();
  }

  /********************************************************************/
  /**
   * CivilianHashtable򤽤Τޤ֤
   *<BR><BR>
   * @return CivilianHashtable
   **/
  public Hashtable getCivilianTable(){
    return objectDB.getCivilianTable();
  }

  /********************************************************************/
  /**
   * FireBrigadeHashtable򤽤Τޤ֤
   *<BR><BR>
   * @return FireBrigadeHashtable
   **/
  public Hashtable getFireBrigadeTable(){
    return objectDB.getFireBrigadeTable();
  }

  /********************************************************************/
  /**
   * AmbulanceTeamHashtable򤽤Τޤ֤
   *<BR><BR>
   * @return AmbulanceTeamHashtable
   **/
  public Hashtable getAmbulanceTeamTable(){
    return objectDB.getAmbulanceTeamTable();
  }

  /********************************************************************/
  /**
   * PoliceForceHashtable򤽤Τޤ֤
   *<BR><BR>
   * @return PoliceForceHashtable
   **/
  public Hashtable getPoliceForceTable(){
    return objectDB.getPoliceForceTable();
  }

  /********************************************************************/
  /**
   * RefugeHashtable򤽤Τޤ֤
   *<BR><BR>
   * @return RefugeHashtable
   **/
  public Hashtable getRefugeTable(){
    return objectDB.getRefugeTable();
  }

  /********************************************************************/
  /**
   * FireStationHashtable򤽤Τޤ֤
   *<BR><BR>
   * @return FireStationHashtable
   **/
  public Hashtable getFireStationTable(){
    return objectDB.getFireStationTable();
  }

  /********************************************************************/
  /**
   * AmbulanceCenterHashtable򤽤Τޤ֤
   *<BR><BR>
   * @return AmbulanceCenterHashtable
   **/
  public Hashtable getAmbulanceCenterTable(){
    return objectDB.getAmbulanceCenterTable();
  }

  /********************************************************************/
  /**
   * PoliceOfficeHashtable򤽤Τޤ֤
   *<BR><BR>
   * @return PoliceOfficeHashtable
   **/
  public Hashtable getPoliceOfficeTable(){
    return objectDB.getPoliceOfficeTable();
  }

  /********************************************************************/
  /**
   * ʪIDʪ (ʪID,׼Ԥ̵ͭ,߽ͥ,׼ID)
   * 
   *<BR><BR>
   * @param id ʪID
   * @return buildingIDǥޥåפƤHashtableξ󥪥֥
   **/
  public Integer[] getCheckBuildingData(int id){
    return objectDB.getCheckBuildingData(id);
  }

  /********************************************************************/
  /**
   * ʪ׼ԾΥϥåơ֥Ф
   *<BR><BR>
   * @return ʪ׼ԾHashtable
   **/
  public Hashtable getCheckBuildingTable(){
    return objectDB.getCheckBuildingTable();
  }

  /********************************************************************/
  // ObjectDatabaseѤ᥽åɷ
  // ޤǡ
  /********************************************************************/
  /********************************************************************/
  // Ϳ줿ID߽(orߵorٻ)ʤtrue֤
  // hasCivilianObject()ϡ̱߽ʤɤäǤtrue֤ؤʤ

  public boolean hasRescueManObject(int id){
    boolean rescueman = false;
    if(objectDB.hasFireBrigadeObject(id)){
      rescueman = true;
    }
    if(objectDB.hasAmbulanceTeamObject(id)){
      rescueman = true;
    }
    if(objectDB.hasPoliceForceObject(id)){
      rescueman = true;
    }
    return rescueman;
  }

  /********************************************************************/
  // Ϳ줿ID(ɽorߵ޽orٻ)ʤtrue֤

  public boolean hasRescueCenterObject(int id){
    boolean rescuecenter = false;
    if(objectDB.hasFireStationObject(id)){
      rescuecenter = true;
    }
    if(objectDB.hasAmbulanceCenterObject(id)){
      rescuecenter = true;
    }
    if(objectDB.hasPoliceOfficeObject(id)){
      rescuecenter = true;
    }
    return rescuecenter;
  }

  /********************************************************************/
  // ߽(orߵorٻ)֤

  public int getRescueManObjectNum(){
    int rescuemanNum = 0;
    rescuemanNum += objectDB.getFireBrigadeObjectNum();
    rescuemanNum += objectDB.getAmbulanceTeamObjectNum();
    rescuemanNum += objectDB.getPoliceForceObjectNum();
    return rescuemanNum;
  }

  /********************************************************************/
  // (ɽorߵ޽orٻ)֤

  public int getRescueCenterObjectNum(){
    int rescuecenterNum = 0;
    rescuecenterNum += objectDB.getFireStationObjectNum();
    rescuecenterNum += objectDB.getAmbulanceCenterObjectNum();
    rescuecenterNum += objectDB.getPoliceOfficeObjectNum();
    return rescuecenterNum;
  }

  /********************************************************************/
  // ȶ̤Υ᥽å
  /********************************************************************/
  /** ʬֶᤤID֤<BR><BR>
   * @param id FB,AT,PF,FS,AC,PO,CV(Civilian)Τ줫IDʳ
   * @return ʬֶᤤIDʾΤ-1֤롣
   **/
  public int getNearestRefugeID(int id){
    int nearestID = -1;
    double distance = -1.0;
    double selfX;
    double selfY;
    // ʬXYɸ򥻥å
    if(objectDB.hasCivilianObject(id)){
      Civilian cv = objectDB.getCivilianObject(id);
      selfX = getPosX(cv);
      selfY = getPosY(cv);
    }
    else if(hasRescueCenterObject(id)){
      Building b = objectDB.getBuildingObject(id);
      selfX = b.getX();
      selfY = b.getY();
    }
    else{
      //System.err.println("IDǤ[getNearestRefugeID()]: ID="+id);
      return -1;
    }
    // ᤤθ
    Enumeration enum = objectDB.getRefugeTable().elements();
    while(enum.hasMoreElements()){
      Refuge rfg = (Refuge)enum.nextElement();
      double d = getDistance(selfX,selfY,rfg.getX(),rfg.getY());
      if(distance < 0 || d < distance){
	distance = d;
	nearestID = rfg.getID();
      }
    }
    //System.out.println("ID:"+id+" ֶᤤID..."+nearestID);
    return nearestID;
  }

  /********************************************************************/
  /** IDʬᤤ֤<BR><BR>
   * @param id FB,AT,PF,FS,AC,PO,CV(Civilian)Τ줫IDʳ
   * @return ʬᤤ¤ID꤬ʤȤnull
   **/
  public int[] getRefugeIDbyNearerOrder(int id){
    double selfX;
    double selfY;
    // ʬXYɸ򥻥å
    if(objectDB.hasCivilianObject(id)){
      Civilian cv = objectDB.getCivilianObject(id);
      selfX = getPosX(cv);
      selfY = getPosY(cv);
    }
    else if(hasRescueCenterObject(id)){
      Building b = objectDB.getBuildingObject(id);
      selfX = b.getX();
      selfY = b.getY();
    }
    else{
      //System.err.println("IDǤ[getRefugeIDbynearerOrder()]: ID="+id);
      return null;
    }
    if(getRefugeObjectNum() == 0){ // ꤬ʤ
      return null;
    }

    // ФȶᤤǤΥ
    Vector refugeSet = getRefugeListWP(); // Ͽޤrefuge
    int count = refugeSet.size();

    // ID(refugeInfo[k][0])ȸϤεΥ(refugeInfo[k][1])
    double[][] refugeInfo = new double[count][2];
    // ID(retInfo[k][0])ȸϤεΥ(retInfo[k][1])
    double[][] retInfo = new double[count][2];
    // ֤ͤǼѿ
    int[] retArray = new int[count];

    Enumeration e = refugeSet.elements();
    int rc = 0;
    while(e.hasMoreElements()){
      Refuge ref = (Refuge)e.nextElement();
      refugeInfo[rc][0] = (double)ref.getID();
      refugeInfo[rc][1] = Math.sqrt((selfX-ref.getX())*(selfX-ref.getX())
				    + (selfY-ref.getY())*(selfY-ref.getY()));
      rc++;
    }

    retInfo = QuickSort.qsort(refugeInfo,0,count-1); // Υ˥

    for(int i=0;i<count;i++){ // ֤٤ʤޤɽ
      retArray[i] = (int)retInfo[i][0];
      //System.out.print(" >" + retArray[i]);
      //System.out.print("(" + retInfo[i][1] + ") ");
    }
    //System.out.println();

    return retArray;
  }

  /********************************************************************/
  // PoliceForce Temporary Method
  /********************************************************************/
  /** 
   * ֥å줿RoadФIDΥꥹȤ֤<BR>
   * (get ID's if it exist blocked road)
   *<BR><BR>
   * @param obj  Ȥʤ뼫ȤΥ֥Ⱦ
   * @return ޤäƤƻϩIDꥹ
   **/

  public int[] getBlockedRoadList(Civilian obj){
    int checkNum = 10; // Ȥꤢ10Ĥޤǵ
    double[][] BroadList = new double[checkNum][2];
    Enumeration enum = objectDB.getRoadTable().elements();

    int num = 0;
    while(enum.hasMoreElements() && num < checkNum){
      Road ro = (Road)enum.nextElement();
      if(ro.getBlock()!=0){
	BroadList[num][0] =  ro.getID();

	// HeadTailΥΡɤ
	Node hnd = objectDB.getNodeObject(ro.getHead());
	Node tnd = objectDB.getNodeObject(ro.getTail());

	// ᤤ
	if(getDist(obj,hnd) < getDist(obj,tnd)){
	  BroadList[num][1] = getDist(obj,hnd);
	}
	else{
	  BroadList[num][1] = getDist(obj,tnd);
	}

	num++;
      }
    }

    if(num != 0){
      double[][] forSort = new double[num][2];
      System.arraycopy(BroadList,0,forSort,0,num);
      double[][] preRetList = QuickSort.qsort(forSort,0,forSort.length-1);

      int[] retList = new int[preRetList.length];
      for(int i=0;i<preRetList.length;i++){
	retList[i] = (int)preRetList[i][0];
      };

      return retList;
    }
    else{
      return null;
    }
  }

  /********************************************************************/
  /**
   *֥å줿RoadФIDΥꥹȤ֤(̤뤫ɤ)<BR>
   * (get ID's if it exist blocked road)
   *<BR><BR>
   * @param obj  Ȥʤ뼫ȤΥ֥Ⱦ
   * @return ޤäƤƻϩIDꥹ
   **/

  public int[] getBlockedRoadListWW(Civilian obj){
    int checkNum = 10; // Ȥꤢ10Ĥޤǵ
    double[][] BroadList = new double[checkNum][2];
    Enumeration enum = objectDB.getRoadTable().elements();

    int num = 0;
    while(enum.hasMoreElements() && num < checkNum){
      Road ro = (Road)enum.nextElement();
      if(checkBlocked(ro)){
	BroadList[num][0] =  ro.getID();

	// HeadTailΥΡɤ
	Node hnd = objectDB.getNodeObject(ro.getHead());
	Node tnd = objectDB.getNodeObject(ro.getTail());

	// ᤤ
	if(getDist(obj,hnd) < getDist(obj,tnd)){
	  BroadList[num][1] = getDist(obj,hnd);
	}
	else{
	  BroadList[num][1] = getDist(obj,tnd);
	};

	num++;
      };
    };

    if(num != 0){
      double[][] forSort = new double[num][2];
      System.arraycopy(BroadList,0,forSort,0,num);
      double[][] preRetList = QuickSort.qsort(forSort,0,forSort.length-1);

      int[] retList = new int[preRetList.length];
      for(int i=0;i<preRetList.length;i++){
	retList[i] = (int)preRetList[i][0];
      };

      return retList;
    }
    else{
      return null;
    }
  }

  /********************************************************************/
  /** 
   * ƻꤵ줿ʾ Road  ID ֤<BR>
   *<BR><BR>
   * @param width ƻ(mm)
   * @return Vector ꤵ줿ʾ Road
   **/

  public Vector getWideRoad(int width){
    Vector ret = new Vector();
    Enumeration enum = objectDB.getRoadTable().elements();
    
    while(enum.hasMoreElements()){
      Road ro = (Road)enum.nextElement();
      if (ro.getWidth() >= width){
	ret.add(new Integer(ro.getID()));
      }
    }

    return ret;
  }

  /********************************************************************/
  /**
   *  PointObjectCivilianεΥ֤<BR>
   *  (Get distance between PointPbject and Civilian)
   *<BR><BR>
   * @param cv Ȥʤ륪֥Ⱦ
   * @param nd оݤȤʤ륪֥Ⱦ
   * @return cv-nd֤εΥ[m]
   **/

  public double getDist(Civilian cv,PointObject nd){
    if(cv == null || nd == null)
      return 99999;

    double cvX = getPosX(cv);
    double cvY = getPosY(cv);

    return Math.sqrt((cvX - nd.getX()) * (cvX - nd.getX())
		     + (cvY - nd.getY()) * (cvY - nd.getY()));
  }

  /********************************************************************/
  /**
   * Ϥ줿Road̹ԲǽȽꡣ<BR>
   * (Check that input Road Object is throughable or not)
   *<BR><BR>
   * @param road Ĵ٤Road֥
   * @return true=ĺ, false=̹Բǽ
   **/

  public boolean checkBlocked(Road road){
    double linesBlockedRate = 
      (double)road.getBlock() / (double)road.getWidth();
    int linesBlocked = (int)Math.floor(linesBlockedRate + 0.5);
    // ȤꤢlinesToHead
    if(road.getLinesToHead() - linesBlocked > 0
       && road.getLinesToTail() - linesBlocked > 0)
      return false; // ̤
    else
      return true; // ֥å̤ʤ
  }

  /********************************************************************/
  /**
   * Ϥ줿IDΥ֥Ⱦ<BR>
   * (Get object information which input ID)
   *<BR><BR>
   * @param ObjectID ߤ֥ȾID
   * @return ֥Ⱦ
   **/

  public RescueObject getObject(int ObjectID){
    return objectDB.getObject(ObjectID);
  }

  /********************************************************************/
  /**
   *  Ϥ줿IDRoadǳޤäƤtrue<BR>
   *  (If input ID is Road and it blocked, return true)
   * <BR><BR>
   * @param targetID ޤäƤ뤫ɤĴ٤륪֥ȤID
   * @return ޤäƤ->true,ޤäƤʤ->false
   **/

  public boolean isTargetBlocked(int targetID){
    if(!objectDB.hasRoadObject(targetID)){
      if(objectDB.hasNodeObject(targetID)){
	Node n = objectDB.getNodeObject(targetID);
	int[] edge = n.getEdge();
	for(int i=0;i<edge.length;i++){
	  if(objectDB.hasRoadObject(targetID)){
	    Road r = objectDB.getRoadObject(targetID);
	    if(!r.isThroughable())
	      return true;
	  };
	};
	return false;
      }
      else{
	  //System.err.println("ObjectManger.isTargetBlocked target not Road " + 
	  //	   targetID);
	if(targetID<0){
	  Throwable t = new Throwable();
	  t.printStackTrace();
	};
	return false;
      }
    }
    else{
      Road r = objectDB.getRoadObject(targetID);
      if(!r.isThroughable())
	return true;
      else
	return false;
    }
  }

  /********************************************************************/
  /**
   *  ĺɤƤƻå<BR>
   * (Check blocked route)
   * <BR><BR>
   * @param rt 롼ȤID
   * @return ĺɤƤRoadľޤǤΥ롼ȡ
   **/

  public int[] checkBlockedRoute(int id,int[] rt){
    if(rt == null)
      return null;

    int retCount = 0;
    for(int i=0;i<rt.length;i++){
      if(objectDB.hasRoadObject(rt[i])){
	Road r = objectDB.getRoadObject(rt[i]);
	if(checkBlocked(r)){
	  //System.out.println(id + " checkBlockedRoute out!! " + retCount);
	  break;
	};
      };
      retCount++;
    };

    if(retCount == 0){
      int[] ret = {rt[0]};
      return ret;
    }
    else{
      // 롼ȤκǸ夬Roadʤڤ
      if(retCount>1 && objectDB.hasRoadObject(rt[retCount-1])){
	int[] ret = new int[retCount-1];
	System.arraycopy(rt,0,ret,0,retCount-1);
	//System.err.println("Road Cut Off!!");
	return ret;
      }
      else{
	int[] ret = new int[retCount];
	System.arraycopy(rt,0,ret,0,retCount);
	return ret;
      }
    }
  }

  /********************************************************************/
  /**
   *  ʬưΤɤޤ褿å<BR>
   * (Chech position between now and previous step by input(route))
   * <BR><BR>
   * @param r_array 롼ȤID
   * @param selfPos ʬΤ֤ID
   * @return ʬϤβܤޤ褿͡
   **/
  
  int checkPrevRoute(int[] r_array,int selfPos){
    if(r_array == null){
      return -1;
    };
    
    for(int i=0;i<r_array.length-2;i++){
      if(selfPos == r_array[i]){
	return i;
      };
    };
    return -1;
  }
  
  /********************************************************************/
  /**
   *  routeȺ֤鼡˿ʤ롼Ȥ򻻽
   * <BR><BR>
   * @param route_array 롼ȤID
   * @param nowPos ʬΤ֤ID
   * @return ʬޤãƤʤ롼ȡ
   **/

  public int[] recycleRoute(int[] route_array,int nowPos){
    int prevMove = checkPrevRoute(route_array,nowPos);
    
    if(prevMove >= 0){
      int[] new_route = new int[route_array.length - prevMove -1];
      for(int i=0;i<route_array.length-prevMove-1;i++){
	new_route[i] = route_array[prevMove+i];
      };
      return new_route;
    }
    else{
      return null;
    }
  }

  /********************************************************************/
  /**
   *  ޤäƤͤΥåid ֤
   * <BR><BR>
   * @param time 
   * @return ǽ˸ĤäCivilian
   **/

  public int[] checkBuriedCivilian(int time){
    Vector ret = new Vector();
    Enumeration enum = objectDB.getCivilianTable().elements();

    while(enum.hasMoreElements()){
      Civilian cv = (Civilian)enum.nextElement();
      if(cv.getBuriedness() > 0 && time-cv.getTime()<5){
	ret.add(new Integer(cv.getID()));
      }
    }

    if (ret.size() > 0){
      int[] ret_array = new int[ret.size()];
      int index = 0;
      Iterator it = ret.iterator();
      while(it.hasNext()){
	ret_array[index++] = ((Integer)it.next()).intValue();
      }
      return ret_array;
    }

    return null;
  }
  /********************************************************************/
  /**
   *  ǳƤʪΥå
   * <BR><BR>
   * @param time 
   * @return ǳƤʪ󡥤ʤ null
   **/

  public int[] checkBurningBuilding(int time){
    // ʪ enumeration
    Enumeration enum = objectDB.getBuildingTable().elements();

    // ǳƤʪ
    Vector tmpBurningBuilding = new Vector(); 

    while(enum.hasMoreElements()){
      Building ro = (Building)enum.nextElement();
      if (ro.isBurnning() && time == ro.getTime()){
	// ǳƤ && ߴ¬
	// ID Ͽ
	tmpBurningBuilding.add(new Integer(ro.getID()));
      }
    }

    // 񤭽Ф
    if (tmpBurningBuilding.size() > 0){
      int[] ret = new int[tmpBurningBuilding.size()];
      for (int i=0;i<tmpBurningBuilding.size();i++){
	ret[i] = ((Integer)tmpBurningBuilding.get(i)).intValue();
      }
      return ret;
    }
    return null;
  }

  /********************************************************************/
  // ĺɤξ 5 ƥå̤ΤΤǧ
  //  Υ᥽åɤޤƯƤʤǽꡩĴ(02-11-22)

  public int nextTargetBlocked(int ID,int time){
    if(objectDB.hasRoadObject(ID)){
      Road r = objectDB.getRoadObject(ID);
      if(r.getBlock()>0  && (time-r.getTime()) < 5)
	return ID;
      else
	return -1;
    }
    else if(objectDB.hasNodeObject(ID)){
      Node n = objectDB.getNodeObject(ID);
      int[] edge = n.getEdge();
      for(int i=0;i<edge.length;i++){
	if(objectDB.hasRoadObject(edge[i])){
	  Road r = objectDB.getRoadObject(edge[i]);
	  if(r.getBlock()>0 && (time-r.getTime()) < 5)
	    return edge[i];	
	};
      };
      return -1;
    }
    else{
      return -1;
    }
  }

  /********************************************************************/
  // AmbulanceTeam Temporary Method
  /********************************************************************/
  /**
   * ʬλäƤRefugeΥꥹȤ֤(Vector)<BR>
   * (Return list of refuge objects(Vector version))
   *  <BR><BR>
   * @return Refuge֥ȤΥꥹ(int list of refuge objects)
   **/ 

  public Vector getRefugeListWP(){
    return new Vector(objectDB.getRefugeTable().values());
  }

  /********************************************************************/
  /**
   * ʬλäƤRefugeIDΥꥹȤ֤<BR>
   * (Return int list of refuge object's ID)
   *  <BR><BR>
   * @return RefugeIDꥹ(int list of refuge object's ID)
   **/ 

  public int[] getRefugeIDList(){
    Object[] refugeList = objectDB.getRefugeTable().values().toArray();
    int[] ret = new int[refugeList.length];
    for(int i=0;i<ret.length;i++){
      ret[i] = ((Refuge)refugeList[i]).getID();
    };

    return ret;
  }

  /********************************************************************/
  /**
   * Ϥ줿Road֥Ȥ֤
   * ŪˤϡRoadΣüNode֥Ȥκɸ顢
   * ׻֤<BR>
   *  <BR><BR>
   * @param rd RoadObject
   * @return XYɸΥꥹ(double list of the middle point of road)
   **/ 

  public double[] getMiddlePointOfRoad(Road rd){
    if(rd != null){
      double[] middleXY = new double[2];
      Node nd1 = getNodeObject(rd.getHead());
      Node nd2 = getNodeObject(rd.getTail());
      double distance = getDistance(nd1.getX(),nd1.getY(),
				    nd2.getX(),nd2.getY());
      middleXY[0] = calcPosX(nd1.getX(),nd1.getY(),
			     nd2.getX(),nd2.getY(),(distance/2));
      middleXY[1] = calcPosY(nd1.getX(),nd1.getY(),
			     nd2.getX(),nd2.getY(),(distance/2));
      //System.out.print("RoadID:"+rd.getID());
      //System.out.print(", Head:(x,y)=("+nd1.getX()+","+nd1.getY()+")");
      //System.out.print(", Tail:(x,y)=("+nd2.getX()+","+nd2.getY()+")");
      //System.out.println(", Middle:(x,y)=("+middleXY[0]+","+middleXY[1]+")");
      return middleXY;
    }
    return null;
  }

  /********************************************************************/
  /**
   * Ϥ줿Road֥ȤλposEXΥ줿κɸ֤
   * ŪˤϡRoadΣüNode֥Ȥκɸ顢
   * ꤵ줿׻֤<BR>
   *  <BR><BR>
   * @param rd RoadObject
   * @param posEX RoadλεΥ
   * @return ꤷXYɸΥꥹ
   *         (double list of the specified point of road)
   **/
  public double[] getSpecifiedPointOfRoad(Road rd, double posEX){
    if(rd != null){
      double[] spPointXY = new double[2];
      Node nd1 = getNodeObject(rd.getHead());
      Node nd2 = getNodeObject(rd.getTail());
      double distance = getDistance(nd1.getX(),nd1.getY(),
				    nd2.getX(),nd2.getY());
      double distancePoint = distance/2; // ǥեȤ
      if(posEX > distance){
	//System.err.print("ʵΥǤ[getSpecifiedPointOfRoad()]: ");
	//System.err.println("Υ..."+distance+"ϵΥ..."+posEX);
	//System.err.println("ҤäȤ餳ä!?ϵΥ..."
	  //	   +(posEX/rd.getLength())*distance);
      }
      else{
	distancePoint = posEX;
	//System.err.println("Υ..."+distance+"ϵΥ..."+posEX);
	//System.err.println("ҤäȤ餳ä!?ϵΥ..."
	//	   +(posEX/rd.getLength())*distance);
      }

      spPointXY[0] = calcPosX(nd1.getX(),nd1.getY(),
			     nd2.getX(),nd2.getY(),distancePoint);
      spPointXY[1] = calcPosY(nd1.getX(),nd1.getY(),
			     nd2.getX(),nd2.getY(),distancePoint);
      //System.out.print("RoadID:"+rd.getID());
      //System.out.print(", Head:(x,y)=("+nd1.getX()+","+nd1.getY()+")");
      //System.out.print(", Tail:(x,y)=("+nd2.getX()+","+nd2.getY()+")");
      //System.out.println(", Point:(x,y)=("+spPointXY[0]+","+spPointXY[1]+")");
      return spPointXY;
    }
    return null;
  }

  /********************************************************************/
  /**
   * Ϥ줿ʪ֥(MovelessObject)IDκɸ濴Ȥơ
   * Ϥ줿Υ(ñ[m])Ⱦ¤ȤˤʪVector֤<BR>
   * (Return int list of building objects)
   *  <BR><BR>
   * @param id MovelessObjectID
   * @param posEX idRoadλHeadεΥ
   * @param distance Υ
   * @return BuildingVector(Vector list of building objects)
   **/
  public Vector getBuildingWithinCircle(int id, double posEX, double distance){

    // Ϥ줿IDMovelessObjectλ¹ 
    if(getObject(id) != null){
      if(getObject(id) instanceof MovelessObject){
	Vector targetB = new Vector();
	MovelessObject mlo = (MovelessObject)getObject(id);
	double spPointX;
	double spPointY;
	//System.out.println("ŪObject̾: "+mlo.getClass().getName());
	if(mlo instanceof PointObject){ // ĤޤRoadʳ
	  spPointX = ((PointObject)mlo).getX();
	  spPointY = ((PointObject)mlo).getY();
	}
	else if(mlo instanceof Road){ // RoadΤȤ
	  double[] spPointXY = getSpecifiedPointOfRoad((Road)mlo,posEX);
	  spPointX = spPointXY[0];
	  spPointY = spPointXY[1];
	}
	else{
	  //System.err.println("ͽ۳MovelessObject: "+mlo.getClass().getName());
	  return null;
	}
	//System.out.print("ObjectID:"+mlo.getID());
	//System.out.print(",(x,y)=("+spPointX+","+spPointY+")");
	//System.out.println(",Distance:"+distance+"[m]");
	//System.out.println("*** Target Building ID and Coordinate ***");
	// ʪ enumeration
	Enumeration enum = objectDB.getBuildingTable().elements();
	while(enum.hasMoreElements()){
	  Building b = (Building)enum.nextElement();
	  double bx = b.getX();
	  double by = b.getY();
	  if(getDistance(spPointX,spPointY,bx,by) <= distance){
	    targetB.add(b);
	    //System.out.print("ID:"+b.getID()+",(x,y)=("+bx+","+by+")");
	    //System.out.println(",Distance:"+getDistance(spPointX,spPointY,bx,by)+"[m]");
	  }
	}
	//System.out.println("*****************************************");
	return targetB;
      }
      else{ // MovelessObjectǤʤ
	//System.err.println("MovelessObjectʳObject");
      }
    }
    // MovelessObjectʳΤȤnull֤
    return null;
  }

  /********************************************************************/
  /**
   * Ϥ줿ʪ֥(MovelessObject)IDκɸ濴Ȥơ
   * Ϥ줿ΥȾ¤ȤˤʪIDꥹȤ֤<BR>
   * (Return int list of building object's ID)
   *  <BR><BR>
   * @param id MovelessObjectID
   * @param posEX idRoadλHeadεΥ
   * @param distance Υ
   * @return BuildingIDꥹ(int list of building object's ID)
   **/ 
  public int[] getBuildingIDListWithinCircle(int id, double posEX,
					     double distance){

    // ᥤ롼ϾΥ᥽åɤǹԤʤΤǡǤϾη
    // 줿ʪ֥ȤIDΥꥹȤˤ֤
    Vector tmpB = getBuildingWithinCircle(id,posEX,distance);
    if(tmpB != null){
      if(tmpB.size() > 0){
	int[] buildingID = new int[tmpB.size()];
	int count = 0;
	Iterator it = tmpB.iterator();
	while (it.hasNext()){
	  Building b = (Building)it.next();
	  buildingID[count] = b.getID();
	  count++;
	}
	return buildingID;
      }
    }
    // MovelessObjectʳΤȤnull֤
    return null;
  }

  /********************************************************************/
  /**
   * Ϥ줿ʪ֥(MovelessObject)IDκɸ濴Ȥơ
   * Ϥ줿ΥȾ¤Ȥˤʪͥ٥ǡ١
   * ƹ롣
   * Τˤϡ
     // ʬ濴ȤȾd[m]ηʪ"help me"ȶǤ
     // ̱ΤǡϰηʪIDꥹȤõʪ
     // ͥHashTableηʪͥ(4)[3(̱ޤäƤ
     // ǽ)]˽롣â˷ʪͥ(4)[4]
     // ʾξϹȤԤʤʤ
   *  <BR><BR>
   **/ 

  public void updateCheckBuildingData(int time,int id,double posEX,double d){

    int[] chkB_List = getBuildingIDListWithinCircle(id,posEX,d);
    for(int i=0; i<chkB_List.length; i++){
      Integer[] data = getCheckBuildingData(chkB_List[i]);
      if(data[3].intValue() <= 3){
	int tmp1 = data[1].intValue();
	int tmp3 = data[3].intValue();
	data[1] = new Integer(time); // ι
	data[3] = new Integer(3); // ʪͥ٤[3]
	// debug_use
	//System.out.println("-----");
	//System.out.println("Building(ID:"+chkB_List[i]+") Database");
	//System.out.println("ID   : "+data[0].intValue());
	//System.out.println("Time : "+tmp1+" -> "+data[1].intValue());
	//System.out.println("Check: "+data[2].intValue());
	//System.out.println("B_Pri: "+tmp3+" -> "+data[3].intValue());
	//System.out.println("R_Pri: "+data[4].intValue());
      }
      else{
	// debug_use
	//System.out.println("-----");
	//System.out.println("*** No Change Database!! ***");
	//System.out.println("Building(ID:"+chkB_List[i]+") Database");
	//System.out.println("ID   : "+data[0].intValue());
	//System.out.println("Time : "+data[1].intValue());
	//System.out.println("Check: "+data[2].intValue());
	//System.out.println("B_Pri: "+data[3].intValue());
	//System.out.println("R_Pri: "+data[4].intValue());
      }
    }
    //System.out.println("-----");
  }

  /********************************************************************/
  /**
   *  Ϥ줿ϩClear(ݲˤĺɤȥȤˤ)
   *  <BR><BR>
   *  @param r      Ĵ٤륪֥IDʥ롼ȡ
   *  @param close  ¾ΥȤ礽Υ롼Ȥ̤ʤȤ뤫
   **/

  public boolean isRouteClear(int[] r){
    //if (isRouteClearByAgent(r,false) && isRouteClearByBlockade(r)){
    if (isRouteClearByBlockade(r)){
      return true;
    }    
    return false;
  }

  /********************************************************************/
  /**
   *  ϩȤˤä̤ʤɤ
   *  <BR><BR>
   *  @param r      Ĵ٤륪֥IDʥ롼ȡ
   *  @param close  ¾ΥȤ礽Υ롼Ȥ̤ʤȤ뤫
   **/

  public boolean isRouteClearByAgent(int[] r,boolean close){
    if(r==null){
      return false;
    }

    // ʬμˤ륨(̱)ΥꥹȤƹ
    Vector agentList = new Vector();
    crtRcvObject = bodyInfoParser.getRcvObjectList();
    Enumeration e = crtRcvObject.elements();
    while(e.hasMoreElements()){
      int igr = ((Integer)e.nextElement()).intValue();
      if(objectDB.hasCivilianObject(igr)){
	Civilian cv = objectDB.getCivilianObject(igr);
	if((cv instanceof FireBrigade || cv instanceof AmbulanceTeam ||
        cv instanceof PoliceForce ) && cv.getID() != getSelfInfo().getID()){
	  agentList.add(cv);
	}
      }
    }

    // TODO: ̩ head->tail,tail->head θ뤳Ȥɬ
    //  kernel λͤǤ̵

    // route 1 ܤĥå
    for(int i=0;i<r.length;i++){
      if(objectDB.hasRoadObject(r[i])){
	Road rd = objectDB.getRoadObject(r[i]);
	int agentNum = 0;
	for (int j=0;j<agentList.size();j++){
	  Civilian cv = (Civilian)agentList.get(j);
	  if (r[i] == cv.getPosition()){
	    agentNum++;
	  }
	}
	if (agentNum >= (rd.getLinesToHead()+rd.getLinesToTail())){
	  // ꥨȤο¿
	  if (close){
	    rd.setBlockForce(999);
	  }
	  return false;
	}
      }
    }
    return true;
  }
  /********************************************************************/
  /**
   *  ϩݲˤĺɤƤ뤫ɤ
   *  <BR><BR>
   *  @param r      Ĵ٤륪֥IDʥ롼ȡ
   **/

  public boolean isRouteClearByBlockade(int[] r){
    if(r==null){
      return false;
    }

    // loop counter i  1 ˤƤΤ
    // ʬΤ꤬̹ԲǽɤĴ٤
    // Թ礬

    for(int i=1;i<r.length;i++){
      if(objectDB.hasRoadObject(r[i])){
	Road rd = objectDB.getRoadObject(r[i]);
	if(!rd.isThroughable()){
	  return false;
	}
      }
    }

    return true;
  }

  /********************************************************************/
  /**
   *  ޤäƤCivilianΥ֤(ꥹ)
   **/

  // !! Civilian饹srcϻ̱ǤϤʤ!!饹
  // ѡ饹ȤƤ!!ä,ºݤˤ,
  // ,ٻ,ߵΤ줫뤳Ȥˤʤ!!
  public int[] getBuriedCivilianListWD(Civilian src,int time){
    final int checkNum = 16; // Ȥꤢ16ͤޤõ
    double[][] list = new double[checkNum][2];
    int listNum = 0;

    double srcX = getPosX(src);
    double srcY = getPosY(src);

    // Civilianε߽
    Enumeration c_enum = objectDB.getCivilianTable().elements();
    while(c_enum.hasMoreElements()){
      Civilian cv = (Civilian)c_enum.nextElement();
      if(cv.getBuriedness() > 0 && cv.getHP() != 0 && 
	 time - cv.getTime() < 5){
	if(listNum >= checkNum) break;
	list[listNum][0] = cv.getID();
	double destX = getPosX(cv);
	double destY = getPosY(cv);
	list[listNum][1] = Math.sqrt((srcX - destX) * (srcX - destX) +
				     (srcY - destY) * (srcY - destY));
	listNum++;
      };
    };

    if(listNum != 0){
      double[][] pret = QuickSort.qsort(list,0,listNum-1); //Υ祽
      int[] ret = new int[listNum];
      for(int i=0;i<listNum;i++){
	ret[i] = (int)pret[i][0];
      };
      //System.arraycopy(list,0,ret,0,listNum);
      return ret;
    }
    else{
      return null; // Ĥʤänull֤    
    }
  }

  /********************************************************************/
  /**
   *  ߤξޤäƤCivilian ID ֤
   *  ToDo : Υäϸ夫
   **/

  public int[] getBuriedOrDamagedCivilian(int pos,int time){
    /*
    final int checkNum = 16; // Ȥꤢ16ͤޤõ
    double[][] list = new double[checkNum][2];
    int listNum = 0;

    double srcX = getPosX(src);
    double srcY = getPosY(src);
    */

    Vector tmp_list = new Vector();

    // Civilianε߽
    Enumeration c_enum = objectDB.getCivilianTable().elements();
    while(c_enum.hasMoreElements()){
      Civilian cv = (Civilian)c_enum.nextElement();
      if((cv.getBuriedness() > 0 || cv.getDamage() > 0) && cv.getHP() != 0 && 
	 time - cv.getTime() == 0 && cv.getPosition() == pos){
	tmp_list.add(new Integer(cv.getID()));
      }
    }

    /*
    if(listNum != 0){
      double[][] pret = QuickSort.qsort(list,0,listNum-1); //Υ祽
      int[] ret = new int[listNum];
      for(int i=0;i<listNum;i++){
	ret[i] = (int)pret[i][0];
      };
      //System.arraycopy(list,0,ret,0,listNum);
      return ret;
    }
    else{
      return null; // Ĥʤänull֤    
    }
    */

    if (tmp_list.size() > 0){
      int[] ret = new int[tmp_list.size()];
      for (int i=0;i<tmp_list.size();i++){
	ret[i] = ((Integer)tmp_list.get(i)).intValue();
      }
      return ret;
    }
    else{
      return null;
    }
  }

  /********************************************************************/
  /**
   *  ߤξˤ AT  ID ֤
   *  ToDo : Υäϸ夫
   **/

  public int[] getSamePosAmbulanceTeam(int pos,int time){
    /*
    final int checkNum = 16; // Ȥꤢ16ͤޤõ
    double[][] list = new double[checkNum][2];
    int listNum = 0;

    double srcX = getPosX(src);
    double srcY = getPosY(src);
    */

    Vector tmp_list = new Vector();

    // Civilianε߽
    Enumeration at_enum = objectDB.getAmbulanceTeamTable().elements();
    while(at_enum.hasMoreElements()){
      AmbulanceTeam at = (AmbulanceTeam)at_enum.nextElement();
      if(time - at.getTime() == 0 && at.getPosition() == pos){
	tmp_list.add(new Integer(at.getID()));
      }
    }

    /*
    if(listNum != 0){
      double[][] pret = QuickSort.qsort(list,0,listNum-1); //Υ祽
      int[] ret = new int[listNum];
      for(int i=0;i<listNum;i++){
	ret[i] = (int)pret[i][0];
      };
      //System.arraycopy(list,0,ret,0,listNum);
      return ret;
    }
    else{
      return null; // Ĥʤänull֤    
    }
    */

    if (tmp_list.size() > 0){
      int[] ret = new int[tmp_list.size()];
      for (int i=0;i<tmp_list.size();i++){
	ret[i] = ((Integer)tmp_list.get(i)).intValue();
      }
      return ret;
    }
    else{
      return null;
    }
  }

  /********************************************************************/
  /**
   * DamageƤǽCivilianõ(ꥹ)
   **/

  public int[] getDamagedCivilianList(){
    int[] list = new int[8];
    int listNum = 0;
    Enumeration enum = objectDB.getCivilianTable().elements();

    while(enum.hasMoreElements()){
      Civilian cv= (Civilian)enum.nextElement();
      // ĤƤ,,ʳˤ̻̱
      // Rescue ȤϼʬȽǤưưˤΤʤ
      if (!(cv instanceof FireBrigade) && !(cv instanceof AmbulanceTeam) && 
	  !(cv instanceof PoliceForce)){
	if(cv.getDamage() > 0 && cv.getHP() != 0 && cv.getBuriedness() == 0 && 
	   !objectDB.hasRefugeObject(cv.getPosition())){
	  list[listNum] = cv.getID();
	  listNum++;
	  if(listNum >= 8) break;
	}
      }
    }
    
    if(listNum != 0){
      int[] ret = new int[listNum];
      System.arraycopy(list,0,ret,0,listNum);
      return ret;
    }
    else{
      return null; // Ĥʤänull֤    
    }
  }

  /********************************************************************/
  /**
   *  ˵߽кȤԤʤƤ뤫ɤ
   **/
  // ToDo:Υ᥽åɡϤää̵̡Ȥषꡣ
  //      Ǥϡɬfalse֤褦ˤƤ롣(04-04-27)

  public boolean checkAlreadyWork(int targetID){
      //    Enumeration enum = objectDB.getAmbulanceTeamTable().elements();
      //
      //    while(enum.hasMoreElements()){
      //      AmbulanceTeam at = (AmbulanceTeam)enum.nextElement();
      //      if(at.getPosition() == targetID)
      //	return true;
      //    };
      //
      //    return false;
    return false;
  }

  /********************************************************************/
  /**
   *  ߵIDꥹȤκ (2004-04-27 maked)
   **/

  public int[] getAmbulanceTeamList(){
    int[] tmp = new int[30]; // ǿŬ(ߵ⤬30ʾˤʤȴ!?)
    int count = 0;

    Enumeration enum = objectDB.getAmbulanceTeamTable().keys();
    while(enum.hasMoreElements()){
      Integer d = (Integer)enum.nextElement();
      tmp[count] = d.intValue();
      count++;
    }

    int[] ret = null;
    if (count != 0){
      ret = new int[count];
      System.arraycopy(tmp,0,ret,0,count);
    }
    return ret;
  }

  /********************************************************************/
  /**
   *  ߵޥ󥿡IDꥹȤκ
   **/

  public int[] getAmbulanceCenterList(){
    int[] tmp = new int[8];
    int count = 0;

    Enumeration enum = objectDB.getAmbulanceCenterTable().keys();
    while(enum.hasMoreElements()){
      Integer d = (Integer)enum.nextElement();
      tmp[count] = d.intValue();
      count++;
    }

    int[] ret = null;
    if (count != 0){
      ret = new int[count];
      System.arraycopy(tmp,0,ret,0,count);
    }
    return ret;
  }

  /********************************************************************/
  // FireBrigade Temporary Method
  /********************************************************************/
  /********************************************************************/
  /**
   * idηʪǳƤ뤫
   **/

  public boolean isBurnBuilding(int id){
    if(objectDB.hasBuildingObject(id)){
      Building b = objectDB.getBuildingObject(id);
      if(b.isBurnning())
	return true;
    };

    return false;
  }

  /********************************************************************/
  /**
   * ɬפܿ׻
   **/

  public int calcExtinguishNum(Building b){
    double around = 4.5 * Math.sqrt(b.getAreaGround());
    double tutu = around / 10.0;
    // ­Ψ׻ : res = (òФƤ)/(tutu)
    return (int)tutu;
  }

  /********************************************************************/
  /**
   *  ǳƤʪ֤ʾˤƱ褦Υ᥽åɤ
   *  Υ᥽åɤˤϻʤʥ饹󥰤
   * <BR><BR>
   * @return ǳƤʪΥ֥Ȥ󡥤ʤ null
   **/
  
  public Vector getBurningBuilding(){
    // ʪ enumeration
    Enumeration enum = objectDB.getBuildingTable().elements();
    
    // ǳƤʪ򵭲
    Vector burningBuilding = new Vector(); 
    
    while(enum.hasMoreElements()){
      Building ro = (Building)enum.nextElement();
      if (ro.isBurnning()){
	// ǳƤ -> ID Ͽ
	burningBuilding.add(ro);
      }
    }

    // ǳƤʪ֤ʤ null
    if (burningBuilding.size() > 0){
      return burningBuilding;
    }
    return null;
  }

  /********************************************************************/
  /**
   *  ʬλ볦ˤΥꥹȤ֤
   * <BR><BR>
   * @return ʬλ볦ˤΥꥹ
   **/
  
  public int[] getNearFireBrigadeList(int time){
    // ʪ enumeration
    Enumeration enum = objectDB.getFireBrigadeTable().elements();
    
    // ͤΤβ
    Vector ret_v = new Vector(); 
    
    while(enum.hasMoreElements()){
      FireBrigade fb = (FireBrigade)enum.nextElement();
      if ((fb.getTime() - time) == 0){
	// 볦ˤ
	ret_v.add(new Integer(fb.getID()));
      }
    }

    // ǳƤʪ֤ʤ null
    if (ret_v.size() > 0){
      int[] ret = new int[ret_v.size()];
      for (int i=0;i<ret.length;i++){
	ret[i] = ((Integer)ret_v.get(i)).intValue();
      }
      return ret;
    }
    return null;
  }

  /********************************************************************/
  /**
   *  ID  DIST ˤ Node ֤
   * ToDo : ̤֤,롼ƥ󥰤ν֤᤮Τ
   *  Ȥꤢ 4 Ĥˤ
   **/

  public int[] getNearNodeList(double DIST,int id){
    Vector ret = new Vector();

    if(objectDB.hasNodeObject(id)){ // ʬΤȤNode
      Node n = objectDB.getNodeObject(id);
      double id_x = n.getX();
      double id_y = n.getY();

      // Node å
      Enumeration enum_node = objectDB.getNodeTable().elements();
      while(enum_node.hasMoreElements()){
	Node target_n = (Node)enum_node.nextElement();
	double target_x = target_n.getX();
	double target_y = target_n.getY();
	if(Math.sqrt((id_x-target_x)*(id_x-target_x)+
		     (id_y-target_y)*(id_y-target_y)) < DIST){
	  ret.add(new Integer(target_n.getID()));
	}
      }

      /* ٤Ƥ֤
      Iterator it = ret.iterator();
      int[] ret_array = new int[ret.size()];
      int index = 0;
      while(it.hasNext()){
	Integer tmp_int = (Integer)it.next();
	ret_array[index++] = tmp_int.intValue();
      }
      ٤Ƥ֤ ޤ*/
      
      int[] ret_array = new int[4];
      int term = 4;
      if (ret.size() < 4){
	term = ret.size();
	ret_array = new int[term];
      }
      for (int i=0;i<term;i++){
	ret_array[i] = ((Integer)ret.get(i)).intValue(); 
      }      

      return ret_array;
    }
    else{
      //System.out.println("getNearNodeList error : id is not node");
    }
    return null;
  }

  /********************************************************************/
  /**
   * Ϥ줿IDᤤлIDν˽<BR>
   * Vector Version
   **/

  public int[] getNearBurningBuilding(Civilian self){
    int srcPosID = self.getPosition();
    double srcX = 0;
    double srcY = 0;
    double pointErr = 0; // Roadˤ+
    Enumeration enum = objectDB.getBuildingTable().elements();

    Vector firedBuil = new Vector();

    srcX = getPosX(self);
    srcY = getPosY(self);

    while(enum.hasMoreElements()){
      Building ro = (Building)enum.nextElement();
      if(ro.isBurnning()){ 
	firedBuil.add(ro); // ǳƤӥVector
      };
    };

    // 
    int count = firedBuil.size();
    if(count==0){
      return null;
    }
    else{
      int[] new_ret = sortByDist(srcX,srcY,firedBuil);
      return new_ret;
    }
  }

  /********************************************************************/
  /**
   * Ϥ줿IDᤤлIDν˽(Υ)<BR>
   * Vector Version
   **/

  public int[] getNearBurningBuildingInDL(Civilian self){
    int srcPosID = self.getPosition();
    double srcX = 0;
    double srcY = 0;
    double pointErr = 0; // Roadˤ+
    Enumeration enum = objectDB.getBuildingTable().elements();

    Vector firedBuil = new Vector();

    srcX = getPosX(self);
    srcY = getPosY(self);

    while(enum.hasMoreElements()){
      Building ro = (Building)enum.nextElement();
      if(ro.isBurnningLittle() && getDist(self,ro)<30){ 
	firedBuil.add(ro); // ǳƤӥVector
      };
    };

    // 
    int count = firedBuil.size();
    if(count==0){
      return null;
    }
    else{
      int[] new_ret = sortByDist(srcX,srcY,firedBuil);
      return new_ret;
    }
  }

  /********************************************************************/
  /**
   *  ǳƤӥʬΰ֤ᤤsort
   **/

  public int[] sortByDist(double X,double Y,Vector builSet){
    int count = builSet.size();

    double[][] buildInfo = new double[count][2]; // 2ĤޤȤ᤿
    // ӥID(0)ȤεΥ(1)

    int[] retArray = new int[count];
    double[][] retInfo = new double[count][2]; // 2ĤޤȤ᤿
    // ӥID(0)ȤεΥ(1)

    Enumeration e = builSet.elements();
    int bc = 0;
    while(e.hasMoreElements()){
      Building b = (Building)e.nextElement();
      buildInfo[bc][0] = (double)b.getID();
      buildInfo[bc][1] = Math.sqrt((X-b.getX())*(X-b.getX())
				   + (Y-b.getY())*(Y-b.getY()));
      bc++;
    };

    retInfo = QuickSort.qsort(buildInfo,0,count-1); // Υ˥

    for(int i=0;i<count;i++){ // ֤٤ʤޤɽ
      retArray[i] = (int)retInfo[i][0];
      //System.out.print(" >" + retArray[i]);
      //System.out.print("(" + retInfo[i][1] + ") ");
    };
    //System.out.println();

    return retArray;
  }

  /********************************************************************/
  /**
   *  ʪ礭¤Ѥ
   **/

  public int[] sortByBuildingSize(Vector buildingSet){
    int count = buildingSet.size();

    // ʪ礭Ǽ
    double[][] buildingInfo = new double[count][2];

    // ͤ
    int[] retArray = new int[count];

    // åȤͤ
    double[][] retInfo = new double[count][2];    

    Enumeration e = buildingSet.elements();
    int index = 0;
    while(e.hasMoreElements()){
      Building b = (Building)e.nextElement();
      buildingInfo[index][0] = (double)b.getID();
      buildingInfo[index][1] = (double)b.getAreaTotal();
      index++;
    };

    retInfo = QuickSort.qsort(buildingInfo,0,count-1); // Υ˥

    // ֤٤
    for(int i=0;i<count;i++){
      retArray[i] = (int)retInfo[i][0];
    };

    return retArray;
  }

  /********************************************************************/
  /**
   * ⤷RoadϤRoadʤͤˤHead or TailNode֤
   **/

  public int getNextNode(int pos,int rand){
    if(objectDB.hasRoadObject(pos)){
      Road r = objectDB.getRoadObject(pos);
      if(rand%2==0)
	return r.getHead();
      else
	return r.getTail();
    }
    else{
      return -1;
    }
  }

  /********************************************************************/
  // RoutingTable ط
  /********************************************************************/

  public void makeRouteTable(){
    routingTable.makeTable();
  }

  /********************************************************************/

  public void printRouteHash(){
    routingTable.printHash();
  }

  /********************************************************************/
  /** ϤŪϤޤǤηϩ֤<BR><BR>
   * @param src ߰
   * @param dest Ū
   * @param blocked ϩۤκݤƻϩĺɤθƹۤ뤫ɤ
   *                true...θ, false...θʤ
   * @return ϩ(֥ID)ۤǤʤäȤnull!?
   **/
  public int[] goToDest(int src,int dest,boolean blocked){
    //  PosEx 򳰤ϤƤ餤
    // code ν¿ʤΤǤȤꤢ
    // ToDo ľ٤
    Civilian cv = (Civilian)getSelfInfo();
    return routingTable.goToDest(src,cv.getPositionExtra(),dest,blocked);
  }

  /********************************************************************/
  /** ϤŪϤޤǤηϩ֤<BR><BR>
   * @param src ߰
   * @param dest Ū
   * @param blocked ϩۤκݤƻϩĺɤθƹۤ뤫ɤ
   *                true...θ, false...θʤ
   * @param dont_use_id ڤƤΤǷϩ˻ȤʤRoadID
   * @return ϩ(֥ID)ۤǤʤäȤnull!?
   **/
  public int[] goToDest(int src,int dest,boolean blocked,Vector dont_use_id){
    //  PosEx 򳰤ϤƤ餤
    // code ν¿ʤΤǤȤꤢ
    // ToDo ľ٤
    Civilian cv = (Civilian)getSelfInfo();
    return routingTable.goToDest(src,cv.getPositionExtra(),
				 dest,blocked,dont_use_id);
  }

  /********************************************************************/

  public int[] getRandomWalkRoute(Civilian self){
    return routingTable.getRandomWalkRoute(self);
  }

  /********************************************************************/
  // BodyInfoParser(=SenseInfoParser)
  /********************************************************************/
    /*
  public void parseSenseInfo(int[] data){
    bodyInfoParser.parseSenseInfo(data);
  }
    */
  /********************************************************************/
  /**
   * @fdeprecated 侩Ȥˤ侩Ȥʤ褦ˤ뤳ȡ
   **/
    /*
  public void parseSenseInfo(int[] msg_header, int[] msg_data){
    bodyInfoParser.parseSenseInfo(msg_header, msg_data);
  }
    */
  /********************************************************************/

  public int getTime(){
    return objectDB.getTime();
  }

  /********************************************************************/
  /**
   * @deprecated 侩Ȥˤ侩Ȥʤ褦ˤ뤳ȡ
   **/
  public RescueObject getSelfInfo(){
    return bodyInfoParser.getSelfInfo();
  }

  /********************************************************************/
  /**
   * Road
   **/

  public void printRoadInfo(int ID){
    Road r = objectDB.getRoadObject(ID);
    r.printInfo();
  }

  /********************************************************************/

  public int getObjectTime(int ID){
    RealObject r = (RealObject)objectDB.getObject(ID);
    return r.getTime();
  }

  /********************************************************************/
  // HearInfoParser
  /********************************************************************/
  /**
   * @fdeprecated 侩Ȥˤ侩Ȥʤ褦ˤ뤳ȡ
   **/
    /*
  public void parseHearInfo(int[] buffer,int time){
    hearInfoParser.parseHearInfo(buffer,time);
  }
    */
  /********************************************************************/
  /**
   * å֤
   *  <BR><BR>
   * @fdeprecated 侩Ȥˤ侩Ȥʤ褦ˤ뤳ȡ
   *  @freturn å
   **/
    /*
  public int getMsgCount(){
    return hearInfoParser.getMsgCount();
  }
    */
  /********************************************************************/
    /*
  // TODO : Ȥꤢⲽ
  public String getObjective(int i){
    String tmp = Integer.toString(hearInfoParser.getCommand(i));
    return tmp;
  }
    */
  /********************************************************************/
  /**
   * @fdeprecated 侩Ȥˤ侩Ȥʤ褦ˤ뤳ȡ
   **/
    /*
  public int[] getTarget(int i){
    return hearInfoParser.getTarget(i);
  }
    */
  /********************************************************************/
  /**
   * @fdeprecated 侩Ȥˤ侩Ȥʤ褦ˤ뤳ȡ
   **/
    /*
  public int[] getReceiver(int i){
    return hearInfoParser.getReceiver(i);
  }
    */
  /********************************************************************/
  /**
   * @fdeprecated 侩Ȥˤ侩Ȥʤ褦ˤ뤳ȡ
   **/
    /*
  public void concatMessage(String r){
    hearInfoParser.concatMessage(r);
  }
    */
  /********************************************************************/
  /**
   * @fdeprecated 侩Ȥˤ侩Ȥʤ褦ˤ뤳ȡ
   **/
    /*
  public int getMsgCountV(){
    return hearInfoParser.getMsgCountV();
  }
    */
  /********************************************************************/
  /**
   * @fdeprecated 侩Ȥˤ侩Ȥʤ褦ˤ뤳ȡ
   **/
    /*
  // TODO : Ȥꤢⲽ
  public String getObjectiveV(int i){
    //return hearInfoParser.getObjectiveV(i);
    return null;
  }
    */
  /********************************************************************/
  /**
   * @fdeprecated 侩Ȥˤ侩Ȥʤ褦ˤ뤳ȡ
   **/
    /*
  public int[] getTargetV(int i){
    return hearInfoParser.getTargetV(i);
  }
    */
  /********************************************************************/
  /**
   * ȤRoadĺɤ
   * @deprecated 侩Ȥˤ侩Ȥʤ褦ˤ뤳ȡ
   **/
  public void blockRoadWithAgent(){
    crtRcvObject = bodyInfoParser.getRcvObjectList();
    Enumeration e = crtRcvObject.elements();
    while(e.hasMoreElements()){
      int igr = ((Integer)e.nextElement()).intValue();
      if(objectDB.hasCivilianObject(igr)){
	Civilian cv = objectDB.getCivilianObject(igr);
	if(cv instanceof FireBrigade || cv instanceof AmbulanceTeam){
	  if(objectDB.hasRoadObject(cv.getPosition())){
	    Road r = objectDB.getRoadObject(cv.getPosition());
	    r.setBlockForce(9999);
	  }
	}
      };
    };    
  }

  /********************************************************************/
  /**
   * method information : last modify 2002/02/18 Yoshiki Asai 
   * method synopsis    : road ˤ,ΥŪ˶ᤤ node ֤
   *                      ΥƱ
   * method status      : Դ
   **/

  public int getNearNextNode(int roadID,double selfPosExtra,int rand){
    if(objectDB.hasRoadObject(roadID)){
      Road r = objectDB.getRoadObject(roadID);
      double length = r.getLength();
      /*
      System.out.println("Debug : method getNearNextNode()");
      System.out.println("Node - Road - Node : " + r.getHead() + " - " + roadID + " - " + r.getTail());
      System.out.println("road lenght : " + length);
      System.out.println("pos Extra : " + selfPosExtra);
      */
      if (selfPosExtra < length/2.0){
	//System.out.println("Near Node(Head) : " + r.getHead());
	return r.getHead();
      }
      else if (selfPosExtra == length/2.0){
	if(rand%2==0){
	  /*
	  System.out.println("Near Node(rand) : " + r.getHead());
	  */
	  return r.getHead();
	}
	else{
	  /*
	  System.out.println("Near Node(Tail) : " + r.getTail());
	  */
	  return r.getTail();
	}
      }
      else{
	//System.out.println("Near Node(Tail) : " + r.getTail());
	return r.getTail();
      }
    }
    else{
      return -1;
    }
  }

  /********************************************************************/
  /**
   * method information : last modify 2002/02/18 Yoshiki Asai 
   * method synopsis    :  node ܤ rode , rode ,
   *                      1 ֤
   * method status      : Դ
   **/

  public int getOneRoadExceptSomeRoad(int nodeID,int exceptNodeID){
    if(objectDB.hasNodeObject(nodeID)){
      Node n = objectDB.getNodeObject(nodeID);
      Vector edgeList = new Vector();
      int[] eList = n.getEdge();
      for (int i=0;i<eList.length;i++){
	RescueObject ro = objectDB.getObject(eList[i]);
	if(ro instanceof Road){
	  // <- RoadΤߤVector˳Ǽ(EdgeˤBuilding⤢)
	  Road rd = (Road)ro;
	  if(rd.isThroughable())
	    edgeList.add(new Integer(ro.getID()));
	}
      }

      //  node κ
      edgeList.remove(new Integer(exceptNodeID));

      if (edgeList.size()==0){
	// nodeID ϹԤߤޤ(road  1 Ĥʤ)Ǥ
	return -1;
      }
      else{
	return ((Integer)edgeList.get(getTime()%edgeList.size())).intValue();
      }
    }
    else{
      return -1;
    }
  }

  /********************************************************************/
  /**
   * @deprecated 侩Ȥˤ侩Ȥʤ褦ˤ뤳ȡ
   **/
    /*
  public Vector getRcvObjectList(){
    return bodyInfoParser.getRcvObjectList();
  }
    */
  /********************************************************************/
}
